@@ -42,7 +42,6 @@ __docformat__ = 'google'
 import time
 import atexit
 import threading
-import subprocess
 from datetime import datetime, timezone
 from math import radians, sin, cos, acos, atan2, pi
 
@@ -64,6 +63,7 @@ class Client:
         inbox (pyjs8call.inboxmonitor): Monitors JS8Call inbox messages
         config (pyjs8call.confighandler): Manages JS8Call configuration file
         heartbeat (pyjs8call.hbnetwork): Manages heartbeat outgoing messages
+        idle (pyjs8call.idlemonitor): Monitors the JS8Call idle timeout
         callback (pyjs8call.client.Callbacks): Callback function reference object
         settings (pyjs8call.client.Settings): Configuration setting function reference object
         clean_directed_text (bool): Remove JS8Call callsign structure from incoming messages, defaults to True
@@ -71,10 +71,9 @@ class Client:
         online (bool): Whether the JS8Call application and pyjs8call interface are online
         host (str): IP address matching JS8Call *TCP Server Hostname* setting
         port (int): Port number matching JS8Call *TCP Server Port* setting
-        headless (bool): Run JS8Call headless via xvfb (linux only)
     '''
 
-    def __init__(self, host='127.0.0.1', port=2442, headless=False, config_path=None):
+    def __init__(self, host='127.0.0.1', port=2442, config_path=None):
         '''Initialize JS8Call API client.
 
         Registers the Client.stop function with the atexit module.
@@ -87,24 +86,13 @@ class Client:
         Args:
             host (str): JS8Call TCP address setting, defaults to '127.0.0.1'
             port (int): JS8Call TCP port setting, defaults to 2442
-            headless (bool): Run JS8Call headless via xvfb (linux only), defaults to False
             config_path (str): Non-standard JS8Call.ini configuration file path, defaults to None
 
         Returns:
             pyjs8call.client: Constructed client object
-
-        Raises:
-            ProcessLookupError: JS8Call application is not installed
-            RuntimeError: JS8Call config file section does not exist (likely because JS8Call has not been run after installing)
         '''
-        try:
-            subprocess.check_output(['which', 'js8call'])
-        except subprocess.CalledProcessError as e:
-            raise ProcessLookupError('JS8Call application not installed') from e
-
         self.host = host
         self.port = port
-        self.headless = headless
         self.clean_directed_text = True
         self.monitor_outgoing = True
         self.online = False
@@ -118,6 +106,7 @@ class Client:
         self.time_master = None
         self.inbox = None
         self.heartbeat = None
+        self.idle = None
 
         # delay between setting value and getting updated value
         self._set_get_delay = 0.1 # seconds
@@ -129,16 +118,7 @@ class Client:
         # stop application and client at exit
         atexit.register(self.stop)
         
-        try:
-            self.settings.enable_autoreply_startup()
-            self.settings.disable_autoreply_confirmation()
-            self.settings.enable_transmit()
-        except RuntimeError as e:
-            raise RuntimeError('Try launching JS8Call, configuring audio and CAT interfaces as needed, '
-                               'and then exiting the application normally. When the application '
-                               'exits normally the first time it will initialize the config file.') from e
-
-    def start(self, debugging=False, logging=False):
+    def start(self, headless=False, debugging=False, logging=False):
         '''Start and connect to the the JS8Call application.
 
         Initializes sub-module objects:
@@ -150,25 +130,39 @@ class Client:
         - Time master (see pyjs8call.timemonitor)
         - Heartbeat networking (see pyjs8call.hbnetwork)
         - Inbox master (see pyjs8call.inboxmonitor)
-
-        Spot, window, offset, and outgoing monitors are started automatically.
+        - Idle monitor (see pyjs8call.idlemonitor)
 
         Adds the @TIME group to JS8Call via the config file to enable drift monitor features.
 
+        If logging is enabled the log file will be stored in the current user's *HOME* directory.
+
         Args:
+            headless (bool): Run JS8Call headless via xvfb (Linux only)
             debugging (bool): Print message data to the console, defaults to False
             logging (bool): Print message data to ~/pyjs8call.log, defaults to False
-        '''
-        # enable JS8Call TCP connection
-        self.config.set('Configuration', 'TCPEnabled', 'true')
-        self.config.set('Configuration', 'TCPServer', self.host)
-        self.config.set('Configuration', 'TCPServerPort', str(self.port))
-        self.config.set('Configuration', 'AcceptTCPRequests', 'true')
-        # support pyjs8call.timemonitor features
-        self.config.add_group('@TIME')
-        self.config.write()
 
-        self.js8call = pyjs8call.JS8Call(self, self.host, self.port, headless=self.headless)
+        Raises:
+            RuntimeError: JS8Call config file section does not exist (likely because JS8Call has not been run and configured after installation)
+        '''
+        try:
+            self.settings.enable_autoreply_startup()
+            self.settings.disable_autoreply_confirmation()
+            self.settings.enable_transmit()
+            # enable JS8Call TCP connection
+            self.config.set('Configuration', 'TCPEnabled', 'true')
+            self.config.set('Configuration', 'TCPServer', self.host)
+            self.config.set('Configuration', 'TCPServerPort', str(self.port))
+            self.config.set('Configuration', 'AcceptTCPRequests', 'true')
+            # support pyjs8call.timemonitor features
+            self.config.add_group('@TIME')
+            self.config.write()
+        except RuntimeError as e:
+            raise RuntimeError('Try launching JS8Call, configuring audio and CAT interfaces as needed, '
+                               'and then exiting the application normally. When the application '
+                               'exits normally the first time it will initialize the config file.') from e
+
+        self.js8call = pyjs8call.JS8Call(self, self.host, self.port)
+        self.js8call.start(headless = headless)
         self.online = True
 
         if debugging:
@@ -190,6 +184,7 @@ class Client:
         self.time_master = pyjs8call.TimeMaster(self)
         self.heartbeat = pyjs8call.HeartbeatNetworking(self)
         self.inbox = pyjs8call.InboxMonitor(self)
+        self.idle = pyjs8call.IdleMonitor(self)
         
         self.window.enable_monitoring()
         self.spots.enable_monitoring()
@@ -199,6 +194,7 @@ class Client:
     def stop(self):
         '''Stop all threads, close the TCP socket, and kill the JS8Call application.'''
         self.online = False
+        self.idle.disable_monitoring()
         
         try:
             return self.js8call.stop()
@@ -214,13 +210,16 @@ class Client:
         self.config.write()
         # save settings
         settings = self.js8call.restart_settings()
+        headless = self.js8call.app.headless
+        idle_monitoring_enabled = self.idle.enabled()
 
         # stop
         self.stop()
         time.sleep(0.25)
 
         # start
-        self.js8call = pyjs8call.JS8Call(self, self.host, self.port, headless=self.headless)
+        self.js8call = pyjs8call.JS8Call(self, self.host, self.port)
+        self.js8call.start(headless = headless)
         self.online = True
 
         rx_thread = threading.Thread(target=self._rx)
@@ -230,6 +229,9 @@ class Client:
 
         # restore settings
         self.js8call.reinitialize(settings)
+        # re-enable idle timeout monitoring
+        if idle_monitoring_enabled:
+            self.idle.enable_monitoring()
 
     def restart_when_inactive(self, age=0):
         '''Restart the JS8Call application once there is no outgoing activity.
@@ -239,7 +241,7 @@ class Client:
         See *pyjs8call.js8call.activity()* for more details.
         
         Args:
-            age (int): Maximum age of outgoing activity to consider active, defaults to 0
+            age (int): Maximum age in seconds of outgoing activity to consider active, defaults to 0
         '''
         thread = threading.Thread(target=self._restart_when_inactive, args=(age,))
         thread.daemon = True
@@ -1274,7 +1276,7 @@ class Settings:
         # map integer to text
         speeds = {4:'slow', 0:'normal', 1:'fast', 2:'turbo', 8:'ultra'}
 
-        if int(submode) in speeds:
+        if submode is not None and int(submode) in speeds:
             return speeds[int(submode)]
         else:
             raise ValueError('Invalid submode \'' + str(submode) + '\'')
@@ -1446,10 +1448,41 @@ class Settings:
         callsign = callsign.upper()
 
         if len(callsign) <= 9 and any(char.isdigit() for char in callsign):
-            return self.config.set('Configuration', 'MyCall', callsign)
+            return self._client.config.set('Configuration', 'MyCall', callsign)
         else:
             raise ValueError('callsign must be <= 9 characters in length and contain at least 1 number')
 
+    def get_idle_timeout(self):
+        '''Get JS8Call idle timeout.
+
+        Returns:
+            int: Idle timeout in minutes
+        '''
+        return self._client.config.get('Configuration', 'TxIdleWatchdog', value_type=int)
+
+    def set_idle_timeout(self, timeout):
+        '''Set JS8Call idle timeout.
+
+        The JS8Call idle timeout must be between 5 and 1440 minutes.
+
+        It is recommended that this function be called before calling *client.start()*. If this function is called after *client.start()* then the application will have to be restarted to utilize the new config file settings. See *client.restart()*.
+
+        Args:
+            timeout (int): Timeout to set in minutes
+
+        Returns:
+            int: Current idle timeout in minutes
+
+        Raises:
+            ValueError: Idle timeout must be between 5 and 1440 minutes
+        '''
+        if timeout < 5 or timeout > 1440:
+            raise ValueError('Idle timeout must be between 5 and 1440 minutes')
+
+        self._client.config.set('Configuration', 'TxIdleWatchdog', timeout)
+
+        return self.get_idle_timeout()
+
     def get_station_grid(self, update=False):
         '''Get JS8Call grid square.
 
